#!/bin/bash

# Sangoma Tech Support Script
# April 09 2011 Moises Silva <moy@sangoma.com>     -- initial version
# Oct 05 2012 Tyler Goodlet <tgoodlet@sangoma.com> -- edit
# Oct 26 2012 Tyler Goodlet                        -- initial 'production' release

# This script is aimed to collect information that is useful for
# a tech support engineer in analysing of a customer system and provides
# a basic assesment of the configuration and driver commands 

# NOTE: Try to avoid printing errors, either log them or discard them by sending
# them to /dev/null, some commands are meant to fail in some Linux distributions

# TODO
# Done          - Add dumping for transcoder cards
# Done? (tyler) - Add PBX detection and control flow -> Is this addressed through dir detection?
#               - Add data card detection and dumping
# Done? (tyler) - Add logging checks for certain files (ex. probe for wanpipe mods inserted in the kernel, etc...)
#               - Factor dirs of interest into a config file? -> python to manage?
#               - Create a "copy file" function/interface which performs logging and copying through a single routine -> this could substantially simplify code!
# Done? (tyler) - Add support for Ubuntu, Debian and SuSE (ie: retrieve list of packages installed and check specific distro version)
#               - Add option flags to turn on extra logging for a redmine?

# Debug
# set -x # uncomment for bash debug on
set -E   # subshells and functions inherit ERR trap

# Script Config
debug=0             # turn on extreme error reporting in log file?
linux_extra=0       # turn on extreme linux logging?
overruncheck_wait=5 # seconds to wait between ifconfig calls

# Dir Structure
debugdir="$PWD/support_logger" # ex. $PWD/support_logger
logfile=$debugdir/log.txt
linuxdir=$debugdir/linux
wandir=$debugdir/wanpipe
astdir=$debugdir/asterisk
dahdidir=$debugdir/dahdi
zapdir=$debugdir/zaptel
fsdir=$debugdir/freeswitch
nsgdir=$debugdir/nsg
nbedir=$debugdir/nbe
sngtcdir=$debugdir/sngtc

# Basic Functions #{{{

# Error function
err () { echo -e "ERROR ${0##*/}: $1" 1>&2 && exit 1; }

# Warning function
warn () { echo -e "WARNING ${0##*/}: $1" 1>&2; }

# Logging funciton
log () {
    [[ -z $1 ]] && err "you must pass an argument to the log function!"
    echo -e "$1" >> $logfile 2>&1
}

# Check pipe success
psuccess() {
    case ${PIPESTATUS[0]} in
    0|141) return 0;;
    *) return 1;;
    esac
}

# check for integer type (FYI: bash regex matching operator (=~) only supported > 3.0)
is_int () {

    if [[ $1 =~ [0-9][0-9]*$ ]]; then
        return 0
    else
        return 1
    fi
}

cleanup () {
    # zip it up
    cd "$debugdir/../"
    output="support_logger-$today.zip"
    if zip -r $output $debugdir >> /dev/null; then
        rm -rf $debugdir
        echo -e "<<< Please attach the following file to your support ticket >>>\n"
        echo "$output"
        echo
        echo "The package is located in '$PWD' on your local system"
        echo "Have a nice day."
    else
        echo
        echo "Failed to zip up $debugdir!"
        echo "please zip up $debugdir manually and submit with your ticket."
    fi
}

handle_error() {
    log "FAILED: line $1, exit code $2"
}

handle_seg() {
    log "SEGV: seg fault line $1, exit code $2"
    warn "segfault detected!"
    cleanup
}

# signal handlers
if (( $debug )); then
    # only use this if you want to know in detail every single command which failed
    trap 'handle_error $LINENO $?' ERR
fi
trap 'handle_seg $LINENO $?' SEGV

#}}}
# Parser #{{{
probe_wps () {

    while read line; do
        read wp _ <<< "${line}"
        if [[ $wp =~ wanpipe ]]; then
            active_wps+=" ${wp##*wanpipe}"
        fi
    done < "/proc/net/wanrouter/status"

    [[ -z "$active_wps" ]] && log "wanrouter status says no interfaces connected!? Check physical layer and cabling!"

    # parse confs and hwprobe to drive probing
    for file in /etc/wanpipe/wanpipe*.conf; do

        # ensure there is a proper config set
        [[ -e $file ]] || { log "failed to parse wanpipe configuration '$file'! -> customer may not have configured card using wancfg!\n";  continue;}

        # strip the 'wanpipe' part and just keep the interface number
        span="$(echo $file | sed -e 's/[^0-9]*//g')"
        list=($(awk '
            /PRI/ { 
                ctype = "pri"
                 exit
            }
            /BRI/ {
                ctype = "bri"
                exit
            }
            /FXO.*FXS/ || /ANALOG/ {
                ctype = "analogue"
                exit
            }
            END { print ctype,width }' "$file"))
            ctype=${list[0]}

            [[ $active_wps =~ $span ]] || { log "$file exists but the wanpipe interface is not loaded!?"; continue; }


            wanpipemon -i w${span}g1 -c sc >> $wandir/stats.txt
            wanpipemon -i w${span}g1 -c so >> $wandir/stats.txt
            echo -e "\n" >> $wandir/stats.txt

            # run commands based on card type
            case "$ctype" in
                "pri")
                    log "$file is a $ctype, running wanpipemon-Ta"
                    echo -e "${file}" >> $wandir/wanpipemon-Ta.txt
                    wanpipemon -i w${span}g1 -c Ta >> $wandir/wanpipemon-Ta.txt
                    ;;

                "bri")
                    log "$file is a $ctype skipping wanpipemon-Ta"
                    ;;

                "analogue")
                    log "$file is $ctype running wanpipemon-astats"
                    echo -e "${file}\n" >> $wandir/wanpipemon-astats.txt

                    while read line; do
                        IFS=':' read module mtype _ <<< "${line}"

                        if [[ "$mtype" =~ FX[OS] ]]; then
                            index="${module##*+*0}"
                            echo -e "module ${module}" >> $wandir/wanpipemon-astats.txt
                            wanpipemon -i w${span}g1 -c astats -m ${index} >> $wandir/wanpipemon-astats.txt
                        fi
                    done < "/proc/net/wanrouter/hwprobe_verbose"
                    ;;
                *)
                    log "strange/unknown wanpipe config detected : $file";;
            esac
            unset list
        done

        # print packet stats
        echo -e "logging interface stats at $overruncheck_wait second intervals\n===" >> $wandir/ifconfig.txt
        for i in {1..3}; do
            echo -e "\n\n$i iteration(s)\n===\n" >> $wandir/ifconfig.txt
            ifconfig >> $wandir/ifconfig.txt
            sleep $overruncheck_wait
        done

    return 0
}
#}}}
# Run commands #{{{
run_wp_cmds () {
    # get hwprobe to see what is installed
    if type "wanrouter" > /dev/null; then

        # wanrouter version &> $wandir/version.txt
        wanrouter version | tee -a $wandir/version.txt $logfile &>/dev/null

        if wanrouter hwprobe verbose &> $wandir/hwprobe.txt; then
            wanrouter status &> "$wandir/status.txt"
            probe_wps || { log "failed to probe wanpipe interfaces!"; return 1; }
            return 0
        else
            log "wanrouter hwprobe failed! -> redumping kernel messages!"
            cp /var/log/messages $debugdir/messages >> $logfile 2>&1
            return 1
        fi
    else
        return 1
    fi
}
run_ast_cmds () {
    # asterisk -V &> $astdir/version.txt
    asterisk -V | tee -a $astdir/version.txt $logfile &>/dev/null

    # run ast commands and log error query fails
    result="$(asterisk -rx 'dahdi show channels')"
    if (( $? != 0 )); then
        log "$result" && return 1
    else
        cat <<< "$result" &> $astdir/dahdi-show-channels.txt
        asterisk -rx 'pri show spans' &> $astdir/pri-show-spans.txt
        asterisk -rx 'zap show channels' &> $astdir/zap-show-channels.txt
        # asterisk -rx 'module load chan_zap.so' &> $astdir/module-load-chan_dahdi.txt
        # asterisk -rx 'module load chan_dahdi.so' &> $astdir/module-load-chan_dahdi.txt
    fi
}
#}}}
#Linux Quarantine #{{{

# extra linux logs not normally needed
run_linux_extra () {
    # kernel line
    cat /proc/cmdline &> $linuxdir/kernel-cmdline.txt

    # many problems can be due to a full hard-drive
    mount &> $linuxdir/mount.txt
    df --si &> $linuxdir/disk-usage.txt

    # processes
    ps axu &> $linuxdir/processes.txt

    # devices
    ls -slh /dev/ &> $linuxdir/dev.txt

    # networking
    route &> $linuxdir/route.txt
    ifconfig -a &> $linuxdir/ifconfig.txt
    iptables -L &> $linuxdir/iptables.txt

    if [[ -d /etc/sysconfig/network-scripts ]] ; then
        log "network scripts detected"

        for script in /etc/sysconfig/network-scripts/*; do

            echo "File: $script" >> $linuxdir/network-devices.txt
            cat $script >> $linuxdir/network-devices.txt
            echo -e "\n" >> $linuxdir/network-devices.txt
        done
    fi

    # libraries
    ls "/usr/lib" &> $linuxdir/usr-lib.txt
    ls "/usr/local/lib" &> "$linuxdir/usr-local-lib.txt"
    ls "/usr/lib64" &> $linuxdir/usr-lib64.txt
    ls "/usr/local/lib64" &> "$linuxdir/usr-local-lib64.txt"

    # kernel modules
    find /lib/modules &> $linuxdir/lib-modules.txt

    # common place for sources
    ls /usr/src &> $linuxdir/usr-src.txt
}
#}}}

# START OF SCRIPT
echo
echo "Starting $0..."
echo "Running script as user '$USER'"
echo
[[ "$USER" != "root" ]] && { err "You must run $0 as user 'root'!"; }

echo "The Sangoma support logger is now collecting logs and configs from your system..."

today="$(date "+%Y-%B-%d-%H_%M_%S")"

# make root dir
mkdir -p $debugdir
mkdir -p $linuxdir
log "Beginning $0..."

# Linux Settings Check
# find out which distribution it is
distro_files="redhat-release debian_version arch-release SuSE-release"

# use word splitting...
for f in $distro_files; do
    file="/etc/$f"
    if [[ -f $file ]]; then
        cp "$file" "$linuxdir/distribution.txt" >> "$logfile" 2>&1
        break
    else
        echo 'unknown linux distribution!' > $linuxdir/distribution.txt
        log "Customer is using an unknown Linux which we do not well support!"
    fi
done

# bash version
log "bash version ${BASH_VERSINFO[*]}"

# kernel logging
cp "/var/log/messages" $debugdir/messages >> $logfile 2>&1
cp "/var/log/syslog" $debugdir/syslog >> $logfile 2>&1
cp "/var/log/syslog.1" $debugdir/syslog >> $logfile 2>/dev/null

# check for pci cards
lspci &> $linuxdir/lspci.txt
listpci="$(lspci | grep -i sangoma)"

if [[ -z "$listpci" ]]; then
    log "no 'telephony' cards reported by lspci!"
    listpci="$(lspci | grep -i  'Broadcom.*BCM5723')"

    if [[ -n "$listpci" ]]; then
        log "transcoder card installed : $listpci"
    fi
fi

# Linux name and kernel start options
uname -a | tee -a $linuxdir/uname.txt $logfile &>/dev/null
cat /etc/issue 2>&1 | tee -a $linuxdir/etc-issue.txt | head -n 3 >> $logfile
cat /proc/interrupts &> $linuxdir/proc-interrupts.txt

# kernel ring buffer
dmesg &> $linuxdir/dmesg.txt

# kernel modules loaded and available
lsmod &> $linuxdir/lsmod.txt
listwanmods="$(lsmod | grep "wan")" 
[[ -z "$listwanmods" ]] && log "lsmod | grep 'wan'-> empty, no wanpipe kmods currently inserted!"
cat <<< "$listwanmods" &> $linuxdir/lsmod_grepwan.txt

# installed packages, redhat and debian based
rpm -qa &> "$linuxdir/rpm-qa.txt"
dpkg -l 2>>$logfile &> $linuxdir/dpkg-l.txt

# do crazy extreme linux logging?
if (( $linux_extra )); then
    log "running extreme linux logging!"
    run_linux_extra
fi

# WANPIPE
if type "wanrouter" > /dev/null; then

    mkdir -p $wandir

    cat /var/log/wanrouter &> $wandir/wanrouter-log.txt
    cp -r /etc/wanpipe/interfaces $wandir/etc-wanpipe >> $logfile 2>&1
    cp -r /etc/wanpipe/wanpipe*.conf $wandir/etc-wanpipe >> $logfile 2>&1
    cp -r /etc/wanpipe/wanrouter.rc $wandir/etc-wanpipe >> $logfile 2>&1
    cat /usr/include/wanpipe/wanpipe_version.h &> $wandir/wanpipe_version.h
    cp -r /etc/ppp $wandir/etc-ppp >> $logfile 2>&1

    loglist=($(find /usr /opt /root -maxdepth 5 -name setup_drv_compile.log))
    for log in ${loglist[*]}; do
        # echo "copying $log to $wandir"
        cp -r $log $wandir/drv_compile_log.txt >> $logfile 2>&1
    done

    run_wp_cmds || log "some wanpipe commands failed!"
else
    log "wanrouter command does not seem to exist?"
fi

# Dahdi/Zaptel
if [[ -f "/etc/dahdi/system.conf" ]]; then
    log "Dahdi/Zaptel directories detected"
    mkdir -p $dahdidir
    cp /etc/dahdi/system.conf $dahdidir/ >> $logfile 2>&1
    dahdi_cfg -vvv >> $dahdidir/dahdi_cfg.txt 2>>$logfile
elif [[ -f "/etc/zaptel.conf" ]]; then
    mkdir -p $zapdir
    cp /etc/zaptel.conf $zapdir/ >> $logfile 2>&1
fi

# Asterisk
if [[ -d "/etc/asterisk" ]]; then
    log "Asterisk directories detected"
    mkdir -p $astdir/etc-asterisk
    mkdir -p $astdir/var-log

    # Asterisk files
    cp -r /etc/asterisk/*dahdi* $astdir/etc-asterisk/ >> $logfile 2>&1
    cp -r /etc/asterisk/extensions.conf* $astdir/etc-asterisk/ >> $logfile 2>&1
    cp -r /etc/asterisk/modules* $astdir/etc-asterisk/ >> $logfile 2>&1
    cp -r /etc/asterisk/sangoma_codec* $astdir/etc-asterisk/ >> $logfile 2>&1
    cp -r /etc/asterisk/sip.conf* $astdir/etc-asterisk/ >> $logfile 2>&1
    cp -r /etc/asterisk/woomera* $astdir/etc-asterisk/ >> $logfile 2>&1


    tail -n 50000 /var/log/asterisk/messages 2>>$logfile > $astdir/var-log/messages.txt 
    tail -n 50000 /var/log/asterisk/full 2>>$logfile > $astdir/var-log/full.txt
    tail -n 50000  /var/log/asterisk/sangoma 2>>$logfile > $astdir/var-log/sangoma.txt

    # Asterisk commands
    run_ast_cmds || log "asterisk is probably not started!"
fi

# FreeSWITCH
if [[ -d "/usr/local/freeswitch/conf" ]]; then
    log "Freeswitch directory detected"
    mkdir -p $fsdir
    cp -r "/usr/local/freeswitch/conf" $fsdir/conf >> $logfile 2>&1
    cp -r "/usr/local/freeswitch/log"  $fsdir/log >> $logfile 2>&1
fi

# NSG
if [[ -d "/usr/local/nsg/conf" ]]; then
    log "NSG directory detected"
    mkdir -p $nsgdir
    cp -r "/usr/local/nsg/conf" $nsgdir/conf >> $logfile 2>&1
    cp -r "/dev/shm/nsg/log" $nsgdir/log >> $logfile 2>&1
    nsg_cli -x "version" &> $nsgdir/version.txt
fi

# NBE
if [[ -d "/opt/Netborder/Express/Gateway/config" ]]; then
    log "NBE directory detected"
    mkdir -p $nbedir
    cp -r "/opt/Netborder/Express/Gateway/config" $nbedir/config >> $logfile 2>&1
    cp -r "/opt/Netborder/Express/Gateway/logs" $nbedir/logs >> $logfile 2>&1
fi

# Transcoder
if [[ -d "/etc/sngtc" ]]; then
    log "Sngtc directory detected"
    mkdir -p $sngtcdir
    cp -r "/etc/sngtc" $sngtcdir/etc >> $logfile 2>&1
    cp -r "/var/log/sngtc/sngtc_server.log" $sngtcdir/sngtc_server.log >> $logfile 2>&1
    modinfo tg3 &> $sngtcdir/modinfo-tg3.txt 2>>$logfile

    # TODO: add steps from the debug doc on the wiki
fi

# When finished collecting
log "Finished $0 dump\n"
echo "Done."
echo
echo "========"
echo
# clean/package everything up...
cleanup

# wonky tar command (legacy code)
# output="support_logger-$today.tar.gz"
# tar -C $debugdir/.. -czf $output ${debugdir##*/} && rm -r $debugdir
